/*
 * The variable used in DisqusJS
 *
 * DisqusJS Mode
 * @param {string} disqusjs.mode = dsqjs | disqus - Set which mode to use, should store and get in localStorage
 *
 * DisqusJS Config
 * @param {string} disqusjs.config.shortname - The disqus shortname
 * @param {string} disqusjs.config.identifier - The identifier of the page
 * @param {string} disqusjs.config.url - The url of the page
 * @param {string} disqusjs.config.api - Where to get data
 * @param {string} disqusjs.config.apikey - The apikey used to request Disqus API
 * @param {string} disqusjs.config.admin - The disqus forum admin username
 * @param {string} disqusjs.config.adminLabel - The disqus moderator badge text
 *
 * DisqusJS Info
 * @param {string} disqusjs.page.id = The thread id, used at next API call
 * @param {string} disqusjs.page.title - The thread title
 * @param {boolean} disqusjs.page.isClosed - Whether the comment is closed
 * @param {number} disqusjs.page.lenfth - How many comment in this thread
 */

function DisqusJS(config) {
    let disqusjs = [];
    disqusjs.config = config

    // 定义 disqusjs.page，之后会填充 thread id、title 等数据
    disqusjs.page = [];

    /*
     * window.disqus_config - 从 disqusjs.config 获取 Disqus 所需的配置
     */
    window.disqus_config = function () {
        this.page.url = disqusjs.config.url;
        this.page.identifier = disqusjs.config.identifier;
    };

    /*
     * get - 封装 XHR GET
     *
     * @param {string} url
     * @param {string} timeout
     * @param {boolean} async
     * @param {function} success
     * @param {function} error
     *
     * Example:
        get({
            'http://localhost:3000/getData',
            4000,
            true,
            (res) => {
                console.log(res)
            },
            () => {}
        })
     */
    const get = (url, success, error) => {
        let xhr = new XMLHttpRequest()
        xhr.open('GET', encodeURI(url), true);
        xhr.onreadystatechange = () => {
            if (xhr.readyState == 4 && xhr.status == 200) {
                let res = JSON.parse(xhr.responseText)
                success(res)
            }
        };
        xhr.timeout = 4000;
        xhr.ontimeout = (e) => {
            error(e);
        };
        xhr.onerror = (e) => {
            error(e);
        };
        xhr.send();
    }

    // localstorage 操作类
    // 用于持久化某些数据（如 newComment 的评论者的相关信息）

    /*
     * setLS(kwy, value) - 设置 localStorage
     *
     * @param {string} key
     * @param {string} value
     */
    const setLS = (key, value) => {
        try {
            localStorage.setItem(key, value);
        } catch (e) {
        }
    }

    /*
     * getLS(key) - 从 localStorage 条目
     *
     * @param {string} key
     * @return {string} - 返回条目 value 内容
     */
    const getLS = (key) => {
        return localStorage.getItem(key);
    }

    /*
     * dateFormat(date) - 解析 date 为 yyyy-MM-dd hh:mm:ss
     *
     * @param {function} date - 传入 Date 对象
     * @return {string} - 格式化后的日期
     */
    const dateFormat = (date) => {
        let y = date.getFullYear();
        let m = date.getMonth() + 1;
        m = m < 10 ? (`0${m}`) : m;
        let d = date.getDate();
        d = d < 10 ? (`0${d}`) : d;
        let h = date.getHours() + 8;
        h = h < 10 ? (`0${h}`) : h;
        let minute = date.getMinutes();
        minute = minute < 10 ? (`0${minute}`) : minute;
        return `${y}-${m}-${d} ${h}:${minute}`;
    }

    /*
     * loadDisqus() - 加载 Disqus
     */
    function loadDisqus() {
        let d = document;
        let s = d.createElement('script');
        s.src = 'https://' + disqusjs.config.shortname + '.disqus.com/embed.js';
        s.setAttribute('data-timestamp', + new Date());
        (d.head || d.body).appendChild(s);
    }

    function checkDisqus() {
        let domain = ['disqus.com', `${disqusjs.config.shortname}.disqus.com`],
            test = 0,
            success = 0;

        let checker = () => {
            if ((domain.length === test) && (test === success)) {
                setLS('dsqjs_mode', 'disqus')
            } else if (domain.length === test) {
                setLS('dsqjs_mode', 'dsqjs')
            }
        }

        for (let i of domain) {
            ((i) => {
                let img = new Image;
                let timeout = setTimeout(() => {
                    img.onerror = img.onload = null;
                    test++;
                    checker()
                }, 2000);

                img.onerror = () => {
                    clearTimeout(timeout);
                    test++;
                    checker()
                }

                img.onload = () => {
                    clearTimeout(timeout);
                    test++;
                    success++;
                    checker()
                }

                img.src = `https://${i}/favicon.ico?${+(new Date)}`

            })(i);

        }
    }

    function loadDsqjs() {
        (() => {
            let url = `${disqusjs.config.api}3.0/threads/list.json?forum=${disqusjs.config.shortname}&thread=ident:${disqusjs.config.identifier}&api_key=${disqusjs.config.apikey}`;
            /*
             * Disqus API 只支持通过 Thread ID 获取评论列表，所以必须先通过 identifier 获取当前页面 Thread ID
             *
             * API Docs: https://disqus.com/api/docs/threads/list/
             * API URI: /3.0/threads/list.json?forum=[disqus_shortname]&thread=ident:[identifier]&api_key=[apikey]
             */
            get(url, (res) => {
                console.log(res);
                // 如果只返回一条则找到了对应 thread，否则是当前 identifier 不能找到唯一的 thread
                // 如果 thread 不唯一则需要进行初始化
                if (res.code === 0 && res.response.length === 1) {
                    var resp = res.response[0];
                    disqusjs.page = {
                        id: resp.id, // Thread ID
                        title: resp.title, // Thread Title (默认是页面标题)
                        isClosed: resp.isClosed, // 评论是否关闭
                        length: resp.posts // 评论数目
                    };
                    // 获取评论列表
                    getComment()
                } else if (res.code === 0 && res.response.length !== 1) {
                    // 当前页面可能还未初始化（创建 thread）
                } else {
                    // 评论列表加载错误
                }
            }, (e) => {
                // 评论列表加载错误
                console.log(e);
            })
        })()

        /*
         * getComment(cursor) - 获取评论列表
         *
         * @param {string} cursor - 传入 cursor 用于加载更多评论
         */
        let getComment = (cursor) => {
            // 处理传入的 cursor
            if (!cursor) {
                cursor = '';
            } else {
                cursor = `&cursor=${cursor}`;
            }
            /*
             * 获取评论列表
             *
             * API Docs: https://disqus.com/api/docs/posts/list/
             * API URI: /3.0/posts/list.json?forum=[shortname]&thread=[thread id]&api_key=[apikey]
             */
            let url = `${disqusjs.config.api}3.0/posts/list.json?forum=${disqusjs.config.shortname}&thread=${disqusjs.page.id}${cursor}&api_key=${disqusjs.config.apikey}`;
            get(url, (res) => {
                if (res.code === 0 && res.response.length > 0) {
                    // 已获得评论列表
                    renderComment(res.response)
                    if (res.cursor.hasNext) {
                        // 显示 加载更多评论 按钮
                    } else {
                        // 没有更多评论了，注意确保按钮隐藏
                    }
                } else if (res.code === 0 && res.response.length === 0) {
                    // 当前没有评论
                } else {
                    // 评论列表加载错误
                }
            }, (e) => {
                // 评论列表加载错误
                console.log(e);
            })
        }

        /*
         * parseCommentData(data) - 解析评论列表
         *
         * @param {Object} data - 评论列表 JSON
         * @return {Object} - 解析后的评论列表数据
         */
        let parseCommentData = (data) => {
            var topLevelComments = [],
                childComments = [];

            let getChildren = (id) => {
                if (childComments.length === 0) {
                    return null;
                }

                var list = [];
                for (let comment of childComments) {
                    if (comment.parent === id) {
                        list.unshift({
                            comment,
                            author: comment.author.name,
                            isPrimary: comment.author.username === disqusjs.config.admin.toLowerCase(),
                            children: getChildren(+comment.id)
                        });
                    }
                }

                if (list.length) {
                    return list;
                } else {
                    return null;
                }
            }

            data.forEach((comment) => {
                (comment.parent ? childComments : topLevelComments)['push'](comment);
            });

            let commentLists = topLevelComments.map((comment) => {
                return {
                    comment,
                    author: comment.author.name,
                    isPrimary: comment.author.username === disqusjs.config.admin.toLowerCase(),
                    children: getChildren(+comment.id)
                };
            });

            return commentLists;
        }

        /*
         * processData(data) - 处理评论列表
         *
         * @param {Object} data - 解析后的评论列表 JSON
         */
        let processData = (data) => {
            // 处理 Disqus Profile
            if (data.comment.author.profileUrl) {
                /*
                Avatar Element
                <a href="${data.comment.author.profileUrl}">
                    <img src="${data.comment.author.avatar.cache}">
                </a>

                Author Element
                <span class="dsqjs-post-author">
                    <a href="${data.comment.author.profileUrl}" target="_blank" rel="nofollow noopener noreferrer">${data.comment.author.name}</a>
                </span>
                */
                data.comment.avatarEl = `<a href="${data.comment.author.profileUrl}"><img src="${data.comment.author.avatar.cache}"></a>`
                data.comment.authorEl = `<span class="dsqjs-post-author"><a href="${data.comment.author.profileUrl}" target="_blank" rel="nofollow noopener noreferrer">${data.comment.author.name}</a></span>`
            } else {
                data.comment.avatarEl = `<img src="${data.comment.author.avatar.cache}">`
                data.comment.authorEl = `<span class="dsqjs-post-author">${data.comment.author.name}</span>`
            }

            // 处理 Admin Label
            if (data.isPrimary) {
                data.comment.authorEl += `<span class="dsqjs-admin-badge">${disqusjs.config.adminLabel}</span>`;
            }

            // 处理评论嵌套级数
            if (data.children) {
                // 如果有子评论，设置当前评论前套数为 1
                data.nesting = 1;
            }

            return data;
        }

        let renderPostItem = (s) => {
            /*
                <div class="dsqjs-post-item dsqjs-clearfix">
                    <div class="dsqjs-post-avatar">
                        ${s.avatarEl}
                    </div>
                    <div class="dsqjs-post-body">
                        <div class="dsqjs-post-header">
                            ${s.authorEl}
                            <span class="dsqjs-bullet"></span>
                            <span class="dsqjs-meta"><time>${dateFormat(new Date(s.createdAt))}</time></span>
                        </div>
                        <div class="dsqjs-post-content">
                            ${s.message}
                        </div>
                    </div>
                </div>
            */
           let html = `<div class="dsqjs-post-item dsqjs-clearfix"><div class="dsqjs-post-avatar">${s.avatarEl}</div><div class="dsqjs-post-body"><div class="dsqjs-post-header">${s.authorEl} <span class="dsqjs-bullet"></span> <span class="dsqjs-meta"><time>${dateFormat(new Date(s.createdAt))}</time></span></div><div class="dsqjs-post-content">${s.message}</div></div></div>`

           return html;
        }



        /*
         * renderCommentData(data) - 渲染评论列表
         *
         * @param {Object} data - 从 getComment() 获取到的 JSON
         */
        let renderComment = (data) => {
            var html = '';

            data = parseCommentData(data);
            data.map((comment) => {
                comment = processData(comment);
                console.log(comment)
                html += `<li data-id="comment-${comment.comment.id}">${renderPostItem(comment.comment)}${childrenComments(s)}</li>`;
            })

            console.log(html)
        }
    }

    loadDsqjs()
}
